'''
ENGLISH language string literals for PyGPSClient application

Created on 12 Sep 2020

@author: semuadmin
'''

TITLE = "PyGPSClient"
WIKIURL = "https://en.wikipedia.org/wiki/NMEA_0183"
GITHUBURL = "https://github.com/semuconsulting/PyGPSClient"

COPYRIGHTTXT = "\u00A9 SEMU Consulting 2020\nBSD 2 Clause License. All Rights Reserved"

INTROTXT = "Welcome to PyGPSClient!"
INTROTXTNOPORTS = "Welcome to PyGPSClient! Please connect a serial GPS device and restart."

HELPTXT = "Help..About - display About dialog."

ABOUTTXT = "PyGPSClient is a free, open-source NMEA & UBX GPS client application written" \
    +" entirely in Python and tkinter.\n\n" \
    +"Instructions and source code are available on Github at the link below."

# Message text
SETINITTXT = "Settings initialised"
VALERROR = "ERROR! Please correct highlighted entries"
SAVEERROR = "ERROR! File could not be saved to specified directory"
OPENFILEERROR = "ERROR! File could not be opened"
BADJSONERROR = "ERROR! Invalid metadata file"
NMEAVALERROR = "Value error in NMEA message: {}"
SEROPENERROR = "Error opening serial port {}"
NOWEBMAPERROR1 = "Map not available."
NOWEBMAPERROR2 = "mqapikey file not found or invalid."
NOWEBMAPERROR3 = "Unable to download web map."
NOWEBMAPERROR4 = "Check Internet connection."
NOWEBMAPERROR5 = "No satellite fix."
WEBMAPHTTPERROR = 'HTTP error downloading web map. Are you connected to the Internet?'
WEBMAPERROR = 'Error downloading web map'
SAVETITLE = "Select Save Directory"
SELTITLE = "Select File for Import"
WAITNMEADATA = "Waiting for NMEA data..."
WAITUBXDATA = "Waiting for UBX data..."
STOPDATA = "Serial reader process stopped"
NOTCONN = "Not connected"
UBXPOLL = "Polling current UBX configuration..."

# Menu text
MENUFILE = "File"
MENUVIEW = "View"
MENUOPTION = "Options"
MENUSAVE = "Save Settings"
MENULOAD = "Load Settings"
MENUEXIT = "Exit"
MENUCAN = "Cancel"
MENURST = "Reset"
MENUHIDESE = "Hide Settings"
MENUSHOWSE = "Show Settings"
MENUHIDEUBX = "Hide UBX Config"
MENUSHOWUBX = "Show UBX Config"
MENUHIDESB = "Hide Status Bar"
MENUSHOWSB = "Show Status Bar"
MENUHIDECON = "Hide Console"
MENUSHOWCON = "Show Console"
MENUHIDEMAP = "Hide Map"
MENUSHOWMAP = "Show Map"
MENUHIDESATS = "Hide Satellites"
MENUSHOWSATS = "Show Satellites"
MENUUBXCONFIG = "UBX Configuration Dialog"
MENUHOWTO = "How To"
MENUABOUT = "About"
MENUHELP = "Help"

# Button text
BTNPLOT = "PLOT"
BTNSAVE = "Save"
BTNCAN = "Cancel"
BTNRST = "Reset"

# Label text
LBLCTL = "Controls"
LBLSET = "Settings"

# Dialog text
DLGUBXCONFIG = "UBX Configuration"
DLGABOUT = "PyGPSClient"
DLGHOWTO = "How To Use PyGPSCLient"
